﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.CodeDom.Compiler;
using System.Reflection;

namespace howto_compile_and_execute
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Deselect the code.
        private void Form1_Load(object sender, EventArgs e)
        {
            txtCode.Select(0, 0);
        }

        // Compile and execute the code.
        private void btnRun_Click(object sender, EventArgs e)
        {
            txtResults.Clear();
            CodeDomProvider code_provider = CodeDomProvider.CreateProvider("C#");

            // Generate a non-executable assembly in memory.
            CompilerParameters parameters = new CompilerParameters();
            parameters.GenerateInMemory = true;
            parameters.GenerateExecutable = false;

            // Add references used by the code. (This one is used by MessageBox.)
            parameters.ReferencedAssemblies.Add("System.Windows.Forms.dll");

            // Compile the code.
            CompilerResults results =
                code_provider.CompileAssemblyFromSource(parameters, txtCode.Text);

            // If there are errors, display them.
            if (results.Errors.Count > 0)
            {
                foreach (CompilerError compiler_error in results.Errors)
                {
                    txtResults.AppendText(
                        "Line: " + compiler_error.Line + ", " +
                        "Error Number: " + compiler_error.ErrorNumber + ", " +
                        compiler_error.ErrorText + "\n");
                }
            }
            else
            {
                // There were no errors.
                txtResults.Text = "Success!";

                // Get the compiled method and execute it.
                foreach (Type a_type in results.CompiledAssembly.GetTypes())
                {
                    if (!a_type.IsClass) continue;
                    if (a_type.IsNotPublic) continue;

                    // Get a MethodInfo object describing the SayHi method.
                    MethodInfo method_info = a_type.GetMethod("SayHi");
                    if (method_info != null)
                    {
                        // Make the parameter list.
                        object[] method_params = new object[] { "This is the parameter string. Isn't it great?" };

                        // Execute the method.
                        try
                        {
                            DialogResult method_result =
                                (DialogResult)method_info.Invoke(null, method_params);

                            // Display the returned result.
                            MessageBox.Show(method_result.ToString());
                        }
                        catch (Exception ex)
                        {
                            MessageBox.Show(ex.Message + '\n' +
                                ex.InnerException.Message + '\n' +
                                ex.InnerException.StackTrace);
                        }
                    }
                }
            }
        }
    }
}
